==> knoda_0.8.0.orig.tar.gz: knoda-0.8/hk_kdeclasses/hk_kderelationdialog.cpp <==
// ****************************************************************************
// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
// This file is part of the hk_kdeclasses library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file LGPL included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
#include "hk_kderelationdialog.h"
#include "hk_kdedbdesigner.h"
#include <hk_datasource.h>
#include <hk_presentation.h>
#include <qtable.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qcombobox.h>

#include <kapplication.h>
#include <kconfig.h>
#include <klocale.h>



hk_kderelationdialog::hk_kderelationdialog( hk_kdedatasourceframe* master, hk_kdedatasourceframe* slave,hk_kdedbdesigner* parent,hk_kdedbrelation* r,  const char* name, bool modal, WFlags fl )
    : hk_kderelationdialogbase( parent, name, modal, fl )
{
p_masterdsframe=master;
p_slavedsframe=slave;
p_designer=parent;
p_relation=r;
const int w=180;
grid->installEventFilter(this);
ok_button->installEventFilter(this);
cancel_button->installEventFilter(this);
grid->setColumnWidth(0,w);
grid->setColumnWidth(1,w);
grid->horizontalHeader()->setLabel(0,QString::fromUtf8(l2u(
			master->designer()->presentation()
				->unique_datasourcename(master->datasource()->presentationnumber())).c_str()));

grid->horizontalHeader()->setLabel(1,QString::fromUtf8(l2u(
			master->designer()->presentation()
				->unique_datasourcename(slave->datasource()->presentationnumber())).c_str()));


set_stringlist(&p_mastercollist,p_masterdsframe->datasource()->columnnames());
set_stringlist(&p_slavecollist,p_slavedsframe->datasource()->columnnames());
add_row();
//ok_button->setEnabled(false);
set_existing_fields();
connect(grid,SIGNAL(valueChanged(int,int)),this,SLOT(slot_data_changed(int,int)));
  KConfig* c=kapp->config();
  QRect rect(0,0,500,300);
  c->setGroup("Relationdialog");
  QRect g;
  g=c->readRectEntry("Geometry",&rect);
  setGeometry(g);
  int cw=150;
  cw=c->readNumEntry("ColwidthLeft",cw);
  grid->setColumnWidth(0,cw);
  cw=c->readNumEntry("ColwidthRight",cw);
  grid->setColumnWidth(1,cw);
  if (master&&master->designer()->presentation()&&master->designer()->presentation()&&master->designer()->presentation()->presentationtype()==hk_presentation::referentialintegrity)
   integritybox->show();
  else
   integritybox->hide();

QString cascade=i18n("Cascade");
QString restrict=i18n("Restrict");
onupdatefield->insertItem(restrict);
onupdatefield->insertItem(cascade);
ondeletefield->insertItem(restrict);
ondeletefield->insertItem(cascade);
if (r)
  {
     onupdatefield->setCurrentItem(r->updatecascade());
     ondeletefield->setCurrentItem(r->deletecascade());
  }
}



hk_kderelationdialog::~hk_kderelationdialog()
{
    // no need to delete child widgets, Qt does it all for us
}





void hk_kderelationdialog::add_row(void)
{

//cerr <<p_mastercollist<<endl;

QComboTableItem* masteritem=new QComboTableItem(grid,p_mastercollist);
QComboTableItem* slaveitem=new QComboTableItem(grid,p_slavecollist);
grid->setNumRows(grid->numRows()+1);
grid->setItem(grid->numRows()-1,0,masteritem);
grid->setItem(grid->numRows()-1,1,slaveitem);

}





void hk_kderelationdialog::set_stringlist(QStringList* strlist,list<hk_string>* collist)
{
    strlist->clear();
    strlist->append("");
   list<hk_string>::iterator it= collist->begin();
   while (it!=collist->end())
   {
       strlist->append(QString::fromUtf8(l2u((*it)).c_str()));
     ++it;
   }

}





void hk_kderelationdialog::set_existing_fields(void)
{
if (!p_slavedsframe->datasource())return;
if (p_slavedsframe->designer()->presentation()->presentationtype()==hk_presentation::referentialintegrity)
  {
  set_existing_refintegrityfields();
  return;
  }

if (p_slavedsframe->datasource()->depending_on()!=p_masterdsframe->datasource())
   {

      return;
   }
 list<hk_string>* slavefields=p_slavedsframe->datasource()->depending_on_thisfields();
 list<hk_string>* masterfields=p_slavedsframe->datasource()->depending_on_masterfields();
list<hk_string>::iterator slaveit=slavefields->begin();
list<hk_string>::iterator masterit=masterfields->begin();
while (slaveit!=slavefields->end())
	{
		((QComboTableItem*)(grid->item(grid->numRows()-1,0)))->setCurrentItem(textposition(&p_mastercollist,*masterit));
		((QComboTableItem*)(grid->item(grid->numRows()-1,1)))->setCurrentItem(textposition(&p_slavecollist,*slaveit));
		++slaveit;
		if (masterit!=masterfields->end()) ++masterit;
		add_row();
	}
}


void hk_kderelationdialog::set_existing_refintegrityfields(void)
{
list <referentialclass>* rl= p_slavedsframe->datasource()->referenceslist();
list <referentialclass>::iterator rit=rl->begin();
 while (rit!=rl->end())
 {
     if ((*rit).p_masterdatasource==p_masterdsframe->datasource()->name())
     {
       list<dependingclass>::iterator it=(*rit).p_fields.begin();
       while (it!=(*rit).p_fields.end())
       {
	((QComboTableItem*)(grid->item(grid->numRows()-1,0)))->setCurrentItem(textposition(&p_mastercollist,(*it).masterfield));
	((QComboTableItem*)(grid->item(grid->numRows()-1,1)))->setCurrentItem(textposition(&p_slavecollist,(*it).dependingfield));

        ++it;
        add_row();

       }
       onupdatefield->setCurrentItem(((*rit).p_updatecascade?1:0));
       ondeletefield->setCurrentItem(((*rit).p_deletecascade?1:0));
       return;
      }
   ++rit;
 }
}

int hk_kderelationdialog::textposition(QStringList* thelist,const hk_string& txt )
{
return thelist->findIndex(QString::fromUtf8(l2u(txt).c_str()));
}



int hk_kderelationdialog::masterindex(const QString& string)
{
return textposition(&p_mastercollist,u2l(string.utf8().data()));
}




int hk_kderelationdialog::slaveindex(const QString& string)
{
return textposition(&p_slavecollist,u2l(string.utf8().data()));
}




void hk_kderelationdialog::ok_button_clicked(void)
{
  if(p_slavedsframe->designer()->presentation()->presentationtype()==hk_presentation::referentialintegrity)
    set_referentialintegrity();
  else
    set_relation();

accept();
KConfig* c=kapp->config();
  c->setGroup("Relationdialog");
  c->writeEntry("Geometry",geometry());
  c->writeEntry("ColwidthLeft",grid->columnWidth(0));
  c->writeEntry("ColwidthRight",grid->columnWidth(1));

}


void hk_kderelationdialog::set_relation(void)
{
  if (p_slavedsframe->datasource()&&p_slavedsframe->datasource()->depending_on())
    {
       hk_kdedbrelation* r=p_designer->get_relation(p_slavedsframe->datasource()->depending_on(),p_slavedsframe->datasource());
       if (r && r!=p_relation) p_designer->delete_relation(r);

    }


p_slavedsframe->datasource()->clear_depending_fields();
 p_slavedsframe->datasource()->set_depending_on_presentationdatasource(
 		p_masterdsframe->datasource()->presentationnumber(),
		p_slavedsframe->datasource()->depending_on_react_on_data_changes(),
		p_slavedsframe->datasource()->dependingmode()
		);

for (int row=0;row<grid->numRows()-1;++row)
  {
    hk_string mastertxt=u2l(grid->item(row,0)->text().utf8().data());
    hk_string slavetxt=u2l(grid->item(row,1)->text().utf8().data());
    if (mastertxt.size()>0 && slavetxt.size()>0)
        p_slavedsframe->datasource()->add_depending_fields(slavetxt,mastertxt,true);
  }

}

void hk_kderelationdialog::set_referentialintegrity(void)
{
      if (p_relation && p_relation->referentialname().size()>0)
      {
        if (!p_slavedsframe->datasource()->drop_reference( p_relation->referentialname()))
        {
         hk_class::show_warningmessage(hk_class::hk_translate("Error: could not delete old reference"));
         return;
        }
      }
      referentialclass ref;
      ref.p_masterdatasource=p_masterdsframe->datasource()->name();
      //ref.p_name=trim(ref_is_on->asstring());
      for (int row=0;row<grid->numRows()-1;++row)
      {
        dependingclass dep;
        dep.masterfield=u2l(grid->item(row,0)->text().utf8().data());
        dep.dependingfield=u2l(grid->item(row,1)->text().utf8().data());
        if (dep.masterfield.size()>0 && dep.dependingfield.size()>0)
        ref.p_fields.push_back(dep);
      }
      ref.p_updatecascade=(onupdatefield->currentItem()==1);
      ref.p_deletecascade=(ondeletefield->currentItem()==1);
      p_slavedsframe->datasource()->add_reference(ref);
   if (p_relation)
   {
   p_relation->set_referentialintegrity(ref,p_masterdsframe,p_slavedsframe);

   }
}


void hk_kderelationdialog::slot_data_changed(int row, int /*col*/)
{
  if (row==grid->numRows()-1
  	&& grid->item(row,0)
	&&  !grid->item(row,0)->text().isEmpty()
	&& !grid->item(row,1)->text().isEmpty())
  	{
		add_row();
	}

ok_button->setEnabled(grid->item(0,0)&&!grid->item(0,0)->text().isEmpty() &&!grid->item(0,1)->text().isEmpty() );
}





void hk_kderelationdialog::keyPressEvent(QKeyEvent* /*event*/)
{
    /*if (event->key()==Key_Delete)
    {
        delete_rows();
    }*/



}



void hk_kderelationdialog::delete_rows(void)
{
 //cerr <<"hk_kderelationdialog::delete_rows"<<endl;
 int beginrow=grid->selection(0).topRow();
 int endrow=grid->selection(0).bottomRow();
 if (endrow==grid->numRows()-1) --endrow;
if (beginrow>-1&&endrow>=beginrow)
{
QMemArray<int> a(endrow-beginrow+1);
int c=0;//cerr <<"a new ="<<endrow-beginrow+1<<endl;
	for (int i=beginrow;i<=endrow;++i)
	{
	 a[c]=i;//cerr <<"a["<<c<<"]="<<i<<endl;
	 ++c;
	}
	grid->removeRows(a);
}
else
{
//cerr <<"delete_row "<<grid->currentRow()<<endl;
if (grid->currentRow()>-1 &&grid->currentRow()<grid->numRows()-1)
	grid->removeRow(grid->currentRow());
}

}




bool hk_kderelationdialog::eventFilter(QObject* object,QEvent* e)
{
 QKeyEvent* event=dynamic_cast<QKeyEvent*>(e);
 if (event  && event->key()==Qt::Key_Delete && event->type()==QEvent::KeyRelease)
   {
       delete_rows();
       //event->accept();
       return true;
   }
  else return hk_kderelationdialogbase::eventFilter(object,e);
}

==> knoda_0.8.2.orig.tar.gz: knoda-0.8.2/hk_kdeclasses/hk_kderelationdialog.cpp <==
// ****************************************************************************
// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
// This file is part of the hk_kdeclasses library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file LGPL included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
#include "hk_kderelationdialog.h"
#include "hk_kdedbdesigner.h"
#include <hk_datasource.h>
#include <hk_database.h>
#include <hk_presentation.h>
#include <hk_connection.h>
#include <qtable.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qradiobutton.h>
#include <qcombobox.h>
#include <qbuttongroup.h>
#include <qwidgetstack.h>

#include <kapplication.h>
#include <kconfig.h>
#include <klocale.h>



hk_kderelationdialog::hk_kderelationdialog( hk_kdedatasourceframe* master, hk_kdedatasourceframe* slave,hk_kdedbdesigner* parent,hk_kdedbrelation* r,  const char* name, bool modal, WFlags fl )
    : hk_kderelationdialogbase( parent, name, modal, fl )
{
p_masterdsframe=master;
p_slavedsframe=slave;
p_designer=parent;
p_relation=r;
const int w=180;
grid->installEventFilter(this);
ok_button->installEventFilter(this);
cancel_button->installEventFilter(this);
grid->setColumnWidth(0,w);
grid->setColumnWidth(1,w);
QString mastername=QString::fromUtf8(l2u(
			master->designer()->presentation()
				->unique_shortdatasourcename(master->datasource()->presentationnumber())).c_str());
QString slavename=QString::fromUtf8(l2u(
			master->designer()->presentation()
				->unique_shortdatasourcename(slave->datasource()->presentationnumber())).c_str());
grid->horizontalHeader()->setLabel(0,mastername);

grid->horizontalHeader()->setLabel(1,slavename);
QString msg=i18n("All of '%1' and equivalent rows of '%2'").arg(mastername).arg(slavename);

button_equivalent->setText(msg);
set_stringlist(&p_mastercollist,p_masterdsframe->datasource()->columnnames());
set_stringlist(&p_slavecollist,p_slavedsframe->datasource()->columnnames());
add_row();
//ok_button->setEnabled(false);
set_existing_fields();
connect(grid,SIGNAL(valueChanged(int,int)),this,SLOT(slot_data_changed(int,int)));
  KConfig* c=kapp->config();
  QRect rect(0,0,500,300);
  c->setGroup("Relationdialog");
  QRect g;
  g=c->readRectEntry("Geometry",&rect);
  setGeometry(g);
  int cw=150;
  cw=c->readNumEntry("ColwidthLeft",cw);
  grid->setColumnWidth(0,cw);
  cw=c->readNumEntry("ColwidthRight",cw);
  grid->setColumnWidth(1,cw);
  if (master&&master->designer()->presentation()&&master->designer()->presentation()&&master->designer()->presentation()->presentationtype()==hk_presentation::referentialintegrity)
  {  
   detailbox->show();
   detailbox->raiseWidget(0);
  }
  else
  {
   if (master->designer()->presentation()->presentationtype()==hk_presentation::qbe)
   {
   detailbox->show();
   detailbox->raiseWidget(1);
   if (master->datasource())
   {
     button_equivalent->setEnabled(p_slavedsframe->datasource()->database()->connection()->server_supports(hk_connection::SUPPORTS_SQL_JOINS));
   }
   
   
   }
   else
   detailbox->hide();
  }

QString cascade=i18n("Cascade");
QString restrict=i18n("Restrict");
onupdatefield->insertItem(restrict);
onupdatefield->insertItem(cascade);
ondeletefield->insertItem(restrict);
ondeletefield->insertItem(cascade);
if (r)
  {
     onupdatefield->setCurrentItem(r->updatecascade());
     ondeletefield->setCurrentItem(r->deletecascade());
  }
}



hk_kderelationdialog::~hk_kderelationdialog()
{
    // no need to delete child widgets, Qt does it all for us
}





void hk_kderelationdialog::add_row(void)
{

//cerr <<p_mastercollist<<endl;

QComboTableItem* masteritem=new QComboTableItem(grid,p_mastercollist);
QComboTableItem* slaveitem=new QComboTableItem(grid,p_slavecollist);
grid->setNumRows(grid->numRows()+1);
grid->setItem(grid->numRows()-1,0,masteritem);
grid->setItem(grid->numRows()-1,1,slaveitem);

}





void hk_kderelationdialog::set_stringlist(QStringList* strlist,list<hk_string>* collist)
{
    strlist->clear();
    strlist->append("");
   list<hk_string>::iterator it= collist->begin();
   while (it!=collist->end())
   {
       strlist->append(QString::fromUtf8(l2u((*it)).c_str()));
     ++it;
   }

}





void hk_kderelationdialog::set_existing_fields(void)
{
if (!p_slavedsframe->datasource())return;
if (p_slavedsframe->designer()->presentation()->presentationtype()==hk_presentation::referentialintegrity)
  {
  set_existing_refintegrityfields();
  return;
  }

if (p_slavedsframe->datasource()->depending_on()!=p_masterdsframe->datasource())
   {

      return;
   }
 list<hk_string>* slavefields=p_slavedsframe->datasource()->depending_on_thisfields();
 list<hk_string>* masterfields=p_slavedsframe->datasource()->depending_on_masterfields();
list<hk_string>::iterator slaveit=slavefields->begin();
list<hk_string>::iterator masterit=masterfields->begin();
while (slaveit!=slavefields->end())
	{
		((QComboTableItem*)(grid->item(grid->numRows()-1,0)))->setCurrentItem(textposition(&p_mastercollist,*masterit));
		((QComboTableItem*)(grid->item(grid->numRows()-1,1)))->setCurrentItem(textposition(&p_slavecollist,*slaveit));
		++slaveit;
		if (masterit!=masterfields->end()) ++masterit;
		add_row();
	}
button_equivalent->setChecked(p_slavedsframe->datasource()->depending_on_is_left_join());


}


void hk_kderelationdialog::set_existing_refintegrityfields(void)
{
list <referentialclass>* rl= p_slavedsframe->datasource()->referenceslist();
list <referentialclass>::iterator rit=rl->begin();
 while (rit!=rl->end())
 {
     if ((*rit).p_masterdatasource==p_masterdsframe->datasource()->name())
     {
       list<dependingclass>::iterator it=(*rit).p_fields.begin();
       while (it!=(*rit).p_fields.end())
       {
	((QComboTableItem*)(grid->item(grid->numRows()-1,0)))->setCurrentItem(textposition(&p_mastercollist,(*it).masterfield));
	((QComboTableItem*)(grid->item(grid->numRows()-1,1)))->setCurrentItem(textposition(&p_slavecollist,(*it).dependingfield));

        ++it;
        add_row();

       }
       onupdatefield->setCurrentItem(((*rit).p_updatecascade?1:0));
       ondeletefield->setCurrentItem(((*rit).p_deletecascade?1:0));
       return;
      }
   ++rit;
 }
}

int hk_kderelationdialog::textposition(QStringList* thelist,const hk_string& txt )
{
int result= thelist->findIndex(QString::fromUtf8(l2u(txt).c_str()));
return result>0?result:0;
}



int hk_kderelationdialog::masterindex(const QString& string)
{
return textposition(&p_mastercollist,u2l(string.utf8().data()));
}




int hk_kderelationdialog::slaveindex(const QString& string)
{
return textposition(&p_slavecollist,u2l(string.utf8().data()));
}




void hk_kderelationdialog::ok_button_clicked(void)
{
bool result=true;
  if(p_slavedsframe->designer()->presentation()->presentationtype()==hk_presentation::referentialintegrity)
    set_referentialintegrity();
  else
    result=set_relation();

if (result)accept();
else reject();
KConfig* c=kapp->config();
  c->setGroup("Relationdialog");
  c->writeEntry("Geometry",geometry());
  c->writeEntry("ColwidthLeft",grid->columnWidth(0));
  c->writeEntry("ColwidthRight",grid->columnWidth(1));

}


bool hk_kderelationdialog::set_relation(void)
{
  if (p_slavedsframe->datasource()&&p_slavedsframe->datasource()->depending_on())
    {
       hk_kdedbrelation* r=p_designer->get_relation(p_slavedsframe->datasource()->depending_on(),p_slavedsframe->datasource());
       if (r && r!=p_relation) 
       {
       if (!show_yesnodialog(replace_all("%2",
       			replace_all("%1",
       			hk_class::hk_translate("The relation between '%1' and '%2' is in the way. Remove it?"),
       			p_designer->presentation()->unique_shortdatasourcename(p_slavedsframe->datasource()->presentationnumber())),
       			p_designer->presentation()->unique_shortdatasourcename(p_slavedsframe->datasource()->depending_on()->presentationnumber())),true)) return false ;

       p_designer->delete_relation(r);
       }

    }


p_slavedsframe->datasource()->clear_depending_fields();
 p_slavedsframe->datasource()->set_depending_on_presentationdatasource(
 		p_masterdsframe->datasource()->presentationnumber(),
		p_slavedsframe->datasource()->depending_on_react_on_data_changes(),
		p_slavedsframe->datasource()->dependingmode()
		);
p_slavedsframe->datasource()->set_depending_on_is_left_join(button_equivalent->isChecked());
for (int row=0;row<grid->numRows()-1;++row)
  {
    hk_string mastertxt=u2l(grid->item(row,0)->text().utf8().data());
    hk_string slavetxt=u2l(grid->item(row,1)->text().utf8().data());
    if (mastertxt.size()>0 && slavetxt.size()>0)
        p_slavedsframe->datasource()->add_depending_fields(slavetxt,mastertxt,true);
  }
return true;
}

void hk_kderelationdialog::set_referentialintegrity(void)
{
      if (p_relation && p_relation->referentialname().size()>0)
      {
        if (!p_slavedsframe->datasource()->drop_reference( p_relation->referentialname()))
        {
         hk_class::show_warningmessage(hk_class::hk_translate("Error: could not delete old reference"));
         return;
        }
      }
      referentialclass ref;
      ref.p_masterdatasource=p_masterdsframe->datasource()->name();
      //ref.p_name=trim(ref_is_on->asstring());
      for (int row=0;row<grid->numRows()-1;++row)
      {
        dependingclass dep;
        dep.masterfield=u2l(grid->item(row,0)->text().utf8().data());
        dep.dependingfield=u2l(grid->item(row,1)->text().utf8().data());
        if (dep.masterfield.size()>0 && dep.dependingfield.size()>0)
        ref.p_fields.push_back(dep);
      }
      ref.p_updatecascade=(onupdatefield->currentItem()==1);
      ref.p_deletecascade=(ondeletefield->currentItem()==1);
      p_slavedsframe->datasource()->add_reference(ref);
   if (p_relation)
   {
   p_relation->set_referentialintegrity(ref,p_masterdsframe,p_slavedsframe);

   }
}


void hk_kderelationdialog::slot_data_changed(int row, int /*col*/)
{
  if (row==grid->numRows()-1
  	&& grid->item(row,0)
	&&  !grid->item(row,0)->text().isEmpty()
	&& !grid->item(row,1)->text().isEmpty())
  	{
		add_row();
	}

ok_button->setEnabled(grid->item(0,0)&&!grid->item(0,0)->text().isEmpty() &&!grid->item(0,1)->text().isEmpty() );
}





void hk_kderelationdialog::keyPressEvent(QKeyEvent* /*event*/)
{
    /*if (event->key()==Key_Delete)
    {
        delete_rows();
    }*/



}



void hk_kderelationdialog::delete_rows(void)
{
 //cerr <<"hk_kderelationdialog::delete_rows"<<endl;
 int beginrow=grid->selection(0).topRow();
 int endrow=grid->selection(0).bottomRow();
 if (endrow==grid->numRows()-1) --endrow;
if (beginrow>-1&&endrow>=beginrow)
{
QMemArray<int> a(endrow-beginrow+1);
int c=0;//cerr <<"a new ="<<endrow-beginrow+1<<endl;
	for (int i=beginrow;i<=endrow;++i)
	{
	 a[c]=i;//cerr <<"a["<<c<<"]="<<i<<endl;
	 ++c;
	}
	grid->removeRows(a);
}
else
{
//cerr <<"delete_row "<<grid->currentRow()<<endl;
if (grid->currentRow()>-1 &&grid->currentRow()<grid->numRows()-1)
	grid->removeRow(grid->currentRow());
}

}




bool hk_kderelationdialog::eventFilter(QObject* object,QEvent* e)
{
 QKeyEvent* event=dynamic_cast<QKeyEvent*>(e);
 if (event  && event->key()==Qt::Key_Delete && event->type()==QEvent::KeyRelease)
   {
       delete_rows();
       //event->accept();
       return true;
   }
  else return hk_kderelationdialogbase::eventFilter(object,e);
}
